<meta charset="UTF-8">
<meta name="viewport" content="width=device-width, initial-scale=1.0">
<title>LANscape</title>
<link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/css/bootstrap.min.css" rel="stylesheet">
<link href="{{ url_for('static', filename='css/style.css') }}" rel="stylesheet">
<script src="https://kit.fontawesome.com/d0b7f59243.js" crossorigin="anonymous"></script>
<link rel="apple-touch-icon" sizes="180x180" href="/static/img/ico/apple-touch-icon.png">
<link rel="icon" type="image/png" sizes="32x32" href="/static/img/ico/favicon-32x32.png">
<link rel="icon" type="image/png" sizes="16x16" href="/static/img/ico/favicon-16x16.png">
<link rel="manifest" href="/static/lanscape.webmanifest">