<!DOCTYPE html>
<html lang="en">
<head>
    {% include 'core/head.html' %}
    {% include 'core/scripts.html' %}
</head>
<body>
    
    {% block content %}{% endblock %}
    <footer class="div-hide">
        <div class="version">
            v{{app_version}}
            {% if is_local %}
                <span class="text-info">(Local)</span> 
            {% elif update_available %} 
                <span>(Update Available)</span> 
            {% endif %}   
            {% if 'a' in app_version %}
                <span class="text-danger">(Alpha)</span>
            {% elif 'b' in app_version %}
                <span class="text-info">(Beta)</span>
            {% endif %}
        </div>

        <div id="app-actions">
            <a href="/info">
                <i 
                    class="fa-duotone fa-light fa-circle-info"
                    data-bs-toggle="tooltip" 
                    data-bs-placement="top" 
                    title="App info">
                </i>
            </a>

            <a href="/shutdown-ui">
                <i 
                    id="power-button" 
                    class="fa-solid fa-power-off"
                    data-bs-toggle="tooltip" 
                    data-bs-placement="top" 
                    title="Shutdown App">
                </i>
            </a>
        </div>
        
    </footer>
</body>
</html>
